/*
 *    Cross.java:  a java applet tutorial on the vector cross product
 *   
 *    by: David McNamara, A. Alan Middleton, Eric Schiff 1996
 *        Gang Cheng (ported to Tango 10/26/96 for demo)
 * 
 *    allows users to drag either of two vectors around by their tip,
 *	continuously updating their cross product
 *	or, rotate the whole construction
 * 
 *    the author make no claims as to the utility of the comments
*/
/* 	organization:
	 classes for additonal display items
	 classes for the actual "interior"
	 Cross.class
*/
import java.awt.*;
import java.lang.*;
import java.util.*;
import java.applet.*;
import java.net.*;
import java.io.*;
///////// Tango API ////////
import interfaces.AppletBase;
import messages.*;
import consts.*;


/***************************************************************************
 *	define a class which will display the angle between a and b 
	as text within a box
-----------------------------------------------------------------------------*/
class AngleDisplay extends Canvas {
  Dimension ul;  // location of upper left hand corner
  Dimension size; 
  Font anglefont; // font used to write out in
  String label;  // constant part of angle display
  String variable; // variable part of angle display
/*----------------------------------------------------------------------
	constructor   
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/  
  AngleDisplay(Dimension ull,int text,Veector a,Veector b,Pseudovector c){
    anglefont=new java.awt.Font("Symbol",Font.BOLD,text);
		// would like to know a way to write out the symbol phi
    ul=ull;
    label=new String("Angle phi : ");
    variable=String.valueOf((int)c.getangle(a,b));
		// set the value of variable from a,b and their cross product
	

  }
//------------------------------------------------------------------------  
void  AngleDisplayUpdate(Veector a,Veector b,Pseudovector c){
    variable=String.valueOf((int)c.getangle(a,b));
  }
  
void  AngleDisplayDraw(Graphics g){
	// need to initialize the size of the box within the draw routine
	// so that i have access to a Graphics
  if(size==null){                     	// if it hasn't been initiallized
    size=new Dimension();		// set the size of the box from
    g.setFont(anglefont);		// the text it will contain
    FontMetrics fm=g.getFontMetrics();
    size.width=fm.stringWidth(label+variable+"  ");
    size.width+=40;
    size.height=fm.getHeight();
    size.height+=20;
  }
// make black box with white border    
    g.setColor(Color.white);
    g.fillRect(ul.width,ul.height,size.width,size.height);
    g.setColor(Color.black);
    g.fillRect(ul.width+2,ul.height+2,size.width-4,size.height-4);
// draw the strings
    g.setColor(Color.white);
    g.setFont(this.anglefont);
    g.drawString(label+variable.toString(),20,size.height-10);
  }

}
/* --------------------------------------------------------------------
	end of class angle display   
***************************************************************************/
/**************************************************************************
	class ButtonBar:
	make a panel to contain the two buttons which call up frames
	for numeric display
	and info (ie credits) frame
---------------------------------------------------------------------*/
class ButtonBar extends Panel{
  Button crredits,nuumerics;
  MyFrameBase creditframe;
  MyDescriptionPanel creditsinterior;  // let this know about interior of credit frame
  MyFrameBase numericframe;
  MyNumericsPanel numericsinterior; // same for numerics frame
  Color hue;
/*-------------------------------------------------------------------*/
  ButtonBar(Color c){
    super();
    setBackground(c);
    crredits=new Button();
    nuumerics=new Button();
    setLayout(new FlowLayout());
    crredits.setLabel("Description");
    nuumerics.setLabel("Numerical Info.");
    add(crredits);
    add(nuumerics);

  }
/*-------------------------------------------------------------------*/
 public void  update(Veector a,Veector b,Pseudovector c){
    if(numericsinterior!=null)numericsinterior.update(a,b,c);
  }
public boolean handleEvent(java.awt.Event e){
  Object target = e.target;
  
  if(e.id==Event.ACTION_EVENT){ 
    if(target==crredits){
      if(creditframe==null){
	creditsinterior=new MyDescriptionPanel();
	creditframe=new MyFrameBase(creditsinterior);
	creditframe.setTitle("Description");
	creditframe.resize(400,300);
	creditframe.show();
	
      }
      if(!creditframe.isVisible()){
	creditframe.show();
      }
    }
    if(target==nuumerics){
      if(numericframe==null){
	numericsinterior=new MyNumericsPanel();
	numericframe=new MyFrameBase(numericsinterior);
	numericframe.resize(250,500);
	numericframe.show();
      }
      if(!numericframe.isVisible()){
	numericframe.show();
      }
    }
  }
  return super.handleEvent(e);
}

}
/*************************************************************************/
/*   MyFrame base:
	put in some generic properties for frames that are called up 
---------------------------------------------------------------------------*/

class MyFrameBase extends Frame {
  // for now all that i need is a way to get rid of the frame
  MenuBar mb;
  Menu frame;  
  MenuItem framedismiss;
  Panel interior;
  MyFrameBase(Panel arg){
    super();
    mb = new MenuBar();
    
    frame = new Menu("Frame",true);
    mb.add(frame);
    
    framedismiss=new MenuItem("dismiss");
    frame.add(framedismiss);
    setMenuBar(mb);
    interior=arg;
    setLayout(new BorderLayout());
    interior.resize(size().height,size().width);
    add("Center",interior);
    interior.show();
  }
  /*----------------------------------------------------------------------*/
  public boolean handleEvent(Event e){
    if(e.id==Event.WINDOW_DESTROY){
      hide(); 	// when i tried to destroy the frame it brought down the whole
	 	// applet, so just hide it
    }
   return super.handleEvent(e);
  }
/*----------------------------------------------------------------*/
  public boolean action(Event e,Object arg){
    if(e.target instanceof MenuItem){
      MenuItem mi=(MenuItem)(e.target);
      if(mi.getLabel()=="dismiss"){
	this.hide();  // c.f. handleEvent
      }
     
	
    }
    return false;
  }
    
}
/*************************************************************************/
/*  class to be inserted into a FrameBase 
        all it is is a bunch of text
 ------------------------------------------------------------*/
class MyDescriptionPanel extends Panel{
  
  MyDescriptionPanel(){
    super();
  }
 
  public void paint(Graphics g){
    int textheight;
    int current;  // current location of pen

    textheight=g.getFontMetrics().getHeight();
    current=5+textheight;
    g.setColor(Color.black);
    g.drawString("This applet can be used to demonstrate some of the",5,current);
    current=current+3+textheight;
    g.drawString(" properties of the vector cross product.",5,current);
    
    current=current+10+textheight;

    g.drawString("You can drag either vector around by its tip.",5,current);
    current=current+3+textheight;
    g.drawString("Dragging the pointer around anywhere else will cause the",5,current);
    current=current+3+textheight;
    g.drawString("whole construction to rotate.",5,current);
    
    current=current+10+textheight;
    g.drawString("Copyright 1996 David McNamara,Alan Middleton,Eric Schiff",5,current);

    current=current+3+textheight;
    g.drawString("Comments, questions to mcnamara@npac.syr.edu",5,current);
  }
}
/***************************************************************************
	another class for insertion into a FrameBase
	  this one displays descriptions of the vectors
-------------------------------------------------------------------*/
class MyNumericsPanel extends Panel{
  Veector a,b;
  Pseudovector c;
  Font itsfont;
  /*-----------------------------------------------------------------*/
  MyNumericsPanel(){
    super();
    a= new Veector();
    b= new Veector();
    c= new Pseudovector();
    itsfont=new Font("Times",Font.PLAIN,14);
  }
  /*-------------------------------------------------------------------------*/
  public void paint(Graphics g){
    int change=0;  // current location of pen
    g.setFont(itsfont);
    change+=subpaintvector(g,a,0,change);
    change=change+10+g.getFontMetrics().getHeight();
    g.setColor(Color.black);
    g.drawString("Angle: "+String.valueOf(c.getangle(a,b)),5,change);
    change+=5;
    change+=subpaintvector(g,b,0,change);
    change+=4;
    change+=subpaintvector(g,c,0,change);
   
  }
  /*---------------------------------------------------------------*/
  /*  this draws the relevent parameters for a vector a
      starting at location (hor,vert)
      returns the amount that the pen has moved    
----------------------------------------------------------------*/
public int  subpaintvector(Graphics g,Veector a,int hor,int vert){
  int currentv=vert+2; // start pen off a little below vert
  int currenth=hor+2;
  g.setColor(a.hue);  // set color to be the color of the vector 
  
  FontMetrics fm=g.getFontMetrics();// get params of font
  int textheight=fm.getHeight();
 
  currentv=currentv+textheight+5;
  currenth+=5;
  g.drawString(a.name+" :",currenth,currentv); // label
  
  currenth+=5;
  currentv=currentv+textheight+5;

  g.drawString("magnitude: "+String.valueOf(java.lang.Math.sqrt(a.cend2.dot(a.cend2))),
	       currenth,currentv);
  currentv=currentv+textheight+5;
  currenth+=5;
  g.drawString("x-component: "+String.valueOf(a.cend2.x),currenth,currentv);
  currentv=currentv+textheight+5;
  g.drawString("y-component: "+String.valueOf(a.cend2.y),currenth,currentv);
  currentv=currentv+textheight+5;
  g.drawString("z-component: "+String.valueOf(a.cend2.z),currenth,currentv);
  
  g.drawRect(hor+2,vert+2,size().width-1,currentv+2-vert);
  currentv+=2;
return currentv-vert;
}  
  /*----------------------------------------------------------------*/
  /*  update just takes in the (new) values fot the vectors  */
public void update(Veector aa,Veector bb,Pseudovector cc){
  a=aa;
  b=bb;
  c=cc;
  repaint();
}
}
/*---------------------------------------------------------------------
  end of numerics panel
  *******************************************************************/

/**************************************************************** */

/***********************************************************************/
/*         start of actual working classes
***************************************************************/

class Coords                     //element for real coordinates
{                                // in 3-d space
double x,y,z;
  /*-----------------------------------------------*/
Coords()
  {
    this.setCoords(0.0,0.0,0.0);
  }
Coords(double x,double y,double z)
  {
    this.setCoords(x,y,z);
  }
Coords(Coords a)
  {
    this.x=a.x;
    this.y=a.y;
    this.z=a.z;
  }
  /*--------------------------------------------------------------------*/
  /* most of these functions are structured so that you perform an operation
     on the coords, rather than returning the result of the operation
     ---------------------------------------------------------------------*/
void setCoords(double x,double y,double z)
  {
    this.x=x;
    this.y=y;
    this.z=z;
  }
double dot(Coords b)                   //the dot product between this and b
  {
    return(this.x*b.x+this.y*b.y+this.z*b.z);
  }
void scalarmult(double a)              // scalar multiplication of this
  {
    this.x=this.x*a;
    this.y=this.y*a;
    this.z=this.z*a;
  }
void plus(Coords b)
  {
    this.x=this.x+b.x;
    this.y=this.y+b.y;
    this.z=this.z+b.z;
  }

void rotateCoords(double theta,double phi)
       // theta= angle of rotation about x axis
       //  phi=angle of rotation about z axis
  {
    double x,y,z; // auxiliary variables to store original values
    x=this.x;
    y=this.y;
    z=this.z;
    // the usual rotation relations
    this.x=x*java.lang.Math.cos(phi)-
           y*java.lang.Math.sin(phi)*java.lang.Math.cos(theta)+
	   z*java.lang.Math.sin(phi)*java.lang.Math.sin(theta);
    this.y=x*java.lang.Math.sin(phi)+
           y*java.lang.Math.cos(phi)*java.lang.Math.cos(theta)-
	   z*java.lang.Math.cos(phi)*java.lang.Math.sin(theta);
    this.z=z*java.lang.Math.cos(theta)+
           y*java.lang.Math.sin(theta);
  }

Diisp converttoDiisp(Persp p)        // converts the coords to the location on screen that 
  {                                  // this point should be displayed;
   int a,b;
          double Q=(p.pinfin.y-p.A)/(this.y+p.pinfin.y);
  a=p.r.width/2+                              // i have an elegent derivation of
	 (int)(p.mag*                         // this transformation but this 
	       (p.pinfin.x*(p.Qzero-Q)        // margin is too small to contain it
		+this.x*Q));
   b=p.r.height/2-
	 (int)(p.mag*
	       (p.pinfin.z*(p.Qzero-Q)
		+this.z*Q));
   Diisp dum=new Diisp(a,b); 
   return(dum);
  }

}/* end of Coord */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Diisp                                    // fundamental element for on screen
{                                              // displays
  int h,v;  
                                                // i have since found that a class exists
  Diisp()                                       // which has much of this functionality
  {
    this.setDiisp(0,0);
  }
Diisp(int hh,int vv)
  {
    this.setDiisp(hh,vv);
  }
  /*----------------------------------*/
void setDiisp(int hh,int vv)
  {
    this.h=hh;
    this.v=vv;
  }
void setDiisp(Diisp a)
  {
    this.h=a.h;
    this.v=a.v;
  }
Coords converttoCoords(Persp p,Coords norm)    //returns the 3-d coords of the                
  {                                            // point whose projection is this
                                             //and which lies on a plane through
                                              //the origin with normal norm
 
   double xp,yp,zp;
   double hp,vp;
   double D,A,B;

   Coords u=new Coords(-1.0*p.pinfin.x,p.pinfin.y,-1.0*p.pinfin.z);
   hp=(double)(this.h-p.r.width/2)/p.mag;
   vp=(double)(p.r.height/2-this.v)/p.mag;

   D=p.pinfin.y-p.A;
   A=hp-p.pinfin.x*p.Qzero;
   B=vp-p.pinfin.z*p.Qzero;

   if(norm.y!=0.0)
     {
       xp=A*norm.dot(u)/(norm.y*D+norm.z*B+norm.x*A);
       yp=xp*D/A;
       zp=yp*B/D;
     }
  else
    {
      if(norm.x!=0.0)
	{
	  zp=B*norm.dot(u)/(norm.x*A+norm.z*B);
	  yp=zp*D/B;
	  xp=yp*A/D;
	}
      else
	{
	  zp=u.z;
	  yp=zp*D/B;
	  xp=yp*A/D;
	}
    }
Coords dum=new Coords(xp,yp,zp);
u.scalarmult(-1.0);
dum.plus(u);
return(dum);
 }
}/* end of class Diisp */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Persp                // a package for setting up perspective
{
  Coords pinfin;          //the "point at infinity "
  double A;               // distance from viewscreen to origin
  double mag;            // a magnification factor
  Dimension r;          //the on screen size of the applet
  double Qzero;        // just a commonly used number derived from pinfin,A

 Persp()
  {
    this.pinfin=new Coords();
    this.A=0.0;
    this.mag=0.0;
    this.r=new Dimension();
    this.Qzero=0.0;
  }

void setPersp(Coords p,double a,double m,Dimension rr)
  {
    this.pinfin=p;
    this.A=a;
    this.mag=m;
    this.r=rr;
    this.Qzero=(p.y-a)/p.y;
  }
}/* end of Persp  */
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Arrowhead        // a mostly graphical class to put
{                     // arrowheads on vectors
  int arrowh[];  // arrays for horizontal locs of the three corners of Ahead
  int arrowv[];  //            vertical
  Persp vanp;
  Arrowhead()
  {
    int i;
    this.vanp=new Persp();
    arrowh=new int[3];
    arrowv=new int[3];
    for(i=0;i<3;++i)
      {
	arrowh[i]=0;
	arrowv[i]=0;
      }
  }
  void setArrowhead(Diisp d,Persp p)  // takes the on screen location of tip and a perspective
    {
      double a,o,h; // adjacent, opposite, hypotenuse
      double c,s;   // rescaled cosine and sine of the angle
      double angle=3.14159/3.0;  // angle of arrowhead
      this.vanp=p;
      a=(double)(d.h-this.vanp.r.width/2);
      o=(double)(this.vanp.r.height/2-d.v);
      h=java.lang.Math.sqrt(a*a+o*o);
      c=p.mag/20.0*java.lang.Math.cos(angle);
      s=p.mag/20.0*java.lang.Math.sin(angle);

      this.arrowh[0]=d.h;
      this.arrowv[0]=d.v;

      this.arrowh[1]=d.h-(int)(s*a/h+c*o/h);
      this.arrowv[1]=d.v-(int)(c*a/h-s*o/h);

      this.arrowh[2]=d.h-(int)(s*a/h-c*o/h);
      this.arrowv[2]=d.v-(int)(-1.0*c*a/h-s*o/h);
    }
  void drawArrowhead(Graphics g,Color c)
    {
      g.setColor(c);
      g.fillPolygon(this.arrowh,this.arrowv,3);
    }
  

}/* end of class Arrowhead */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Linesegment               // building block for more complicated 
{                               // line like objects
public  Coords cend1,cend2;    // on screen just a single straight line

  Diisp dend1,dend2;
  Color hue;

  Persp vanp;
/*---------------------------------------*/
Linesegment()
{
  this.cend1=new Coords();
  this.cend2=new Coords();
  this.dend1=new Diisp();
  this.dend2=new Diisp();
  this.hue=Color.black;
  this.vanp=new Persp();

}
void setLinesegment(Coords e1,Coords e2,Color shade,Persp vaanp)
  {
    this.cend1=e1;
    this.cend2=e2;

    this.hue=shade;

    this.vanp=vaanp;

    this.dend1=e1.converttoDiisp(vaanp);
    this.dend2=e2.converttoDiisp(vaanp);
  }
void setLinesegment(Coords R,Color shade, Persp vaanp)
{
  Coords origin = new Coords(0.0,0.0,0.0);
  this.cend1=origin;
  this.cend2=R;
  this.hue=shade;
  this.vanp=vaanp;
  this.dend1=origin.converttoDiisp(vaanp);
  this.dend2=R.converttoDiisp(vaanp);
  

}
/*-------------------------------------------------------*/

/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

void setCoords(Coords e1,Coords e2)
  {
    this.cend1=e1;
    this.cend2=e2;
    this.dend1=e1.converttoDiisp(this.vanp);
    this.dend2=e2.converttoDiisp(this.vanp);
  }




void drawLinesegment(Graphics g)
  {
   g.setColor(this.hue);
   g.drawLine(this.dend1.h,this.dend1.v,this.dend2.h,this.dend2.v);
 }
}/*end of class Linesegment */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Veector extends Linesegment
// note: not the most general class, as i know that one end is always attached to the origin
{                                    // my vectors
  Coords normal;  // normal to the plane in which it is constrained to live
  boolean ilive;  // true if this vector is the active one
  Color deadcolor;
  Color livecolor;
  String name;
  Arrowhead ahead;
Veector()
  {
    super();
    this.normal=new Coords();
    this.ilive=false;
    this.name=" ";
    this.deadcolor=Color.black;
    this.livecolor=Color.black;
    this.ahead=new Arrowhead();
  }
void setVeector(Coords R,Color shade,Coords norm,Persp x,String na)
  {
    this.setLinesegment(R,shade,x);
    this.deadcolor=shade;
    this.livecolor=Color.red;
    this.ilive=false;
    this.name=na;
    this.normal=norm;
    this.ahead=new Arrowhead();
    this.ahead.vanp=x;
    this.dend2=this.cend2.converttoDiisp(x);
    this.ahead.setArrowhead(this.dend2,x);
  }
  void setDiispV(int horiz,int vert)  // takes the onscreen location 
                                     //and sets the Coords of this
                                     //and updates its arrowhead
  {
    this.dend2.h=horiz;
    this.dend2.v=vert;
    this.cend2=this.dend2.converttoCoords(this.vanp,this.normal);
    this.ahead.setArrowhead(this.dend2,this.vanp);
  }
void normalizeV()
  {
    double factor;
    
    factor=java.lang.Math.sqrt(this.cend2.dot(this.cend2));
    this.cend2.scalarmult(1.0/factor);
    this.dend2=this.cend2.converttoDiisp(this.vanp);
    this.ahead.setArrowhead(this.dend2,this.vanp);
  }
void rotateV(double theta,double phi)
  {
    this.cend2.rotateCoords(theta,phi);// rotate vector
    this.normal.rotateCoords(theta,phi);

    this.dend2=this.cend2.converttoDiisp(this.vanp);
    this.ahead.setArrowhead(this.dend2,this.vanp);
  }


void waxonV()
  {                         // makes the vector active
    this.ilive=true;
    this.hue=livecolor;
  }
void waxoffV()              // makes the vector inactive
  {
    this.ilive=false;
    this.hue=deadcolor;
  }
void drawV(Graphics g)
  {
    int i,j;
   g.setColor(this.hue);
   for(i=-1;i<2;++i)          // these loops makes the vector thick
      {
	for(j=-1;j<2;++j)
	  {
	    g.drawLine(this.dend1.h+i,this.dend1.v+j,
		       this.dend2.h+i,this.dend2.v+j);
	  }
      }
    this.ahead.drawArrowhead(g,this.hue);
    g.drawString(name,this.dend2.h+8,this.dend2.v+15);

  }/*end of drawV */
  void drawV(Graphics g,Color c) // draw the vector in the color c
                                 // i believe that this is now defunct
                                 // a remanant from earlier attempts at draw routines
  {
    int i,j;
   g.setColor(c);
    for(i=-1;i<2;++i)
      {
	for(j=-1;j<2;++j)
	  {
	    g.drawLine(this.dend1.h+i,this.dend1.v+j,
		       this.dend2.h+i,this.dend2.v+j);
	  }
      }
    this.ahead.drawArrowhead(g,c);
    
    g.drawString(this.name,this.dend2.h+8,this.dend2.v+15);
  }/*end of drawV  with color*/

}/* end of class Veector */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Pseudovector extends Veector
{                                      // a class for the crossproduct
                                       
Pseudovector()
  {
    super();
  }
void setPseudovector(Veector a,Veector b,Color c,String na)
  {
    Coords dumcoords=new Coords(a.cend2);
    this.setVeector(a.cend2,c,dumcoords,a.vanp,na);
    this.cend2=this.crossprod(a,b);
    this.dend2=this.cend2.converttoDiisp(this.vanp);
    this.ahead.setArrowhead(this.dend2,this.vanp);
  }
void updatePV(Veector a,Veector b)
  {
    this.cend2=this.crossprod(a,b);
    this.dend2=this.cend2.converttoDiisp(this.vanp);
    this.ahead.setArrowhead(this.dend2,this.vanp);
	
  }
  Coords crossprod(Veector a,Veector b)          
    {
      Coords dum = new Coords(0.0,0.0,0.0);
      dum.x=a.cend2.y*b.cend2.z-a.cend2.z*b.cend2.y;
      dum.y=a.cend2.z*b.cend2.x-a.cend2.x*b.cend2.z;
      dum.z=a.cend2.x*b.cend2.y-a.cend2.y*b.cend2.x;
      return(dum);
    }
public double getangle(Veector a,Veector b)//c should be axb
  {                                 // returns int approx to angle
    double dum;                     // between a and b
    double idum;
    double PI=3.14159;
    dum=a.cend2.dot(b.cend2);
    dum=dum/java.lang.Math.sqrt(a.cend2.dot(a.cend2));
    dum=dum/java.lang.Math.sqrt(b.cend2.dot(b.cend2));
    idum=(180.0/PI*java.lang.Math.acos(dum));
    if(this.cend2.dot(a.normal)<0) idum*=-1.0;

    return(idum);
  }/* end of get angle */

}/*end of pseudovector*/
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
class Pllane
{                                // a mostly graphical class 
  Persp vanp;                    // to display the plane that 
  Color hue;                     // the vectors live in
  Coords ccorners[]=new Coords[4]; 
  Diisp dcorners[]=new Diisp[4];   
  int icornsh[]=new int[4];       
  int icornsv[]=new int[4];
 
Pllane()
  {
    this.vanp=new Persp();
    this.hue=Color.black;
    int i;
    for(i=0;i<4;++i)
      {
	ccorners[i]=new Coords();
	dcorners[i]=new Diisp();
	icornsh[i]=0;
	icornsv[i]=0;
      }
  }
  void setPllane(Persp p,Color c) // notice fixed size of plane in real space
   {
     int i;
     this.vanp=p;
     this.hue=c;
     ccorners[0].setCoords(-1.2,-1.2,0.0);
     ccorners[1].setCoords(1.2,-1.2,0.0);
     ccorners[2].setCoords(1.2,1.2,0.0);
     ccorners[3].setCoords(-1.2,1.2,0.0);
     for(i=0;i<4;++i) dcorners[i]=ccorners[i].converttoDiisp(p);
     for(i=0;i<4;++i) icornsh[i]=dcorners[i].h;
     for(i=0;i<4;++i) icornsv[i]=dcorners[i].v;
   }
void rotateP(double theta,double phi)
  {
    int i;
    for(i=0;i<4;++i) ccorners[i].rotateCoords(theta,phi);
    for(i=0;i<4;++i) dcorners[i]=ccorners[i].converttoDiisp(this.vanp);
    for(i=0;i<4;++i)
      {
	icornsh[i]=dcorners[i].h;
	icornsv[i]=dcorners[i].v;
      }
  }
void drawP(Graphics g)
  {
    g.setColor(this.hue);
    g.fillPolygon(this.icornsh,this.icornsv,4);
  }
  
}/* end of class Pllane */

public class Cross extends AppletBase
{

/* -----------------------------------------------------------*/
/*  for now just declare the variables; 
    initialize them in init where i can get external attributes
--------------------------------------------------------*/
 
  ButtonBar buttons;
  
  AngleDisplay angbox;

  static final double PI=3.14159; // set PI
  
  // group of things for the double buffered draw routine
  Dimension offdimension;
  Image offimage;
  Graphics offgraphics;
  Graphics clipoff;
    
  Persp perspective=new Persp();
  Coords viewplane=new Coords();// a point that is useful to determine which
                                //objects are infront of one another
  
  Pllane grid=new Pllane();

    Veector a=new Veector();
    Veector b=new Veector();
  
  Pseudovector c=new Pseudovector();

  int globalu,globalv;  // integers to pass around mouse event variables
  int oldgu,oldgv;      // integers to store previous mouse event variables
  Color planecolor,backcolor,bordercolor;

  Color textcolor;
  int textsize;
  Image wallpaper;

// Master-slave status
  boolean 	     Master = true;

public void init(){

  //////// Register to Tango (this must be the first statement) ////
  register();

  // put in the button bar at bottom   
  bordercolor=colorfromstring(getParameter("bordercolor"),Color.black);
  this.setLayout(new BorderLayout());
  buttons = new ButtonBar(bordercolor); 
  buttons.resize(perspective.r.width,50);
  this.add("South",buttons);

  // get the background image
  wallpaper=getImage(getDocumentBase(),new String("background.gif"));
  // setup perspective
  // notice that i assume that things are pretty much in a sphere of radius
  // 1 about the origin in real space
  // also that the magnification factor depends on the height of the applet
  this.perspective.setPersp(new Coords(2.0,10.0,3.5),1.0,
			    (double)(size().height/2),size());
  this.viewplane.setCoords(2.0,-10.0,3.5);// should be mirror reflection 
                                          //through x-z plane of point at 
                                          //infinity
  // set up the plane the vectors live in
  this.grid.setPllane(perspective,
		      colorfromstring(getParameter("planescolor"),Color.gray)
		      );
  
  this.a.setVeector(new Coords(1.0,0.0,0.0),
		    colorfromstring(getParameter("ascolor"),Color.blue),
		    new Coords(0.0,0.0,1.0),
		    perspective,
		    getParameter("asname")
		    );
  
  this.b.setVeector(new Coords(0.0,1.0,0.0),
		    colorfromstring(getParameter("bscolor"),Color.green),
		    new Coords(0.0,0.0,1.0),
		    perspective,
		    getParameter("bsname")
		    );
  
  this.c.setPseudovector(a,b,
			   colorfromstring(getParameter("cscolor"),Color.black),                          getParameter("csname"));
  
  
  //********************
  int x;
  
    if(getParameter("displayangle")==null) angbox=null;
    else{
      try{    
	x=Integer.valueOf(getParameter("angletextsize")).intValue();
      }
      catch(NumberFormatException e){ x=14;}
      angbox=new AngleDisplay(new Dimension(3,3),x,a,b,c);// put ul at 3,3
    }
    
}/*End of init */

public void paint(Graphics g){
  update(g);
}

public void update(Graphics g){
  
  if(offgraphics==null){   // if not initialized initialize it
    offimage=createImage(perspective.r.width,perspective.r.height);
    offgraphics=offimage.getGraphics();
    clipoff=offgraphics.create();
    clipoff.clipRect(3,3,perspective.r.width-6,perspective.r.height-6);
    // clip it so that there is a border arount the whole applet
  }

  offgraphics.setColor(bordercolor);
  offgraphics.fillRect(0,0,perspective.r.width,perspective.r.height);
                                      // fill rect with border color	
                                     //  if wallpaper doesnt work out
                                     //then we have a full rect of border color
  clipoff.drawImage(wallpaper,0,0,perspective.r.width,perspective.r.height,this);
  Font dumfont= new java.awt.Font("Helvetica",Font.BOLD,28);
  offgraphics.setFont(dumfont);
  clipoff.setFont(dumfont);
  
  if(c.cend2.dot(viewplane)<0.0) c.drawV(offgraphics);    // need to determine which
  if(a.cend2.y>b.cend2.y){                               // objects get drawn in which order
    grid.drawP(clipoff);
    a.drawV(clipoff);
    b.drawV(clipoff);
  }
  else{
    grid.drawP(clipoff);
    b.drawV(clipoff);
    a.drawV(clipoff);
    }
  if(c.cend2.dot(viewplane)>=0.0) c.drawV(clipoff);
  if(angbox!=null) angbox.AngleDisplayDraw(clipoff);
 
  g.drawImage(offimage,0,0,this);
  
  buttons.update(a,b,c);
}
 
public boolean mouseDown(java.awt.Event e,int u,int v){
  int dista,distb;

  if (!Master) return true;

  dista=(u-a.dend2.h)*(u-a.dend2.h)+(v-a.dend2.v)*(v-a.dend2.v);
  distb=(u-b.dend2.h)*(u-b.dend2.h)+(v-b.dend2.v)*(v-b.dend2.v);
  
  if(dista<=distb&dista<100) a.waxonV(); // if close to either vector turn it on
  if(distb<dista&distb<100) b.waxonV();

  oldgu=u;
  oldgv=v;
  repaint();
  sendData(0,u,v);
  return true;
}

public boolean mouseUp(java.awt.Event e,int u, int v){
  if (!Master) return true;

  a.waxoffV();  // turn off both vectors
  b.waxoffV();
  repaint();
  buttons.update(a,b,c);
  sendData(1,u,v);
  return true;
}
public boolean mouseDrag(java.awt.Event e,int u,int v){
  double theta,phi;

  if (!Master) return true;

  if(a.ilive|b.ilive){  // update the active one
    Diisp dumdiisp= new Diisp(u,v);
    Coords dumcoords = new Coords(dumdiisp.converttoCoords(perspective,a.normal));
    
    theta=java.lang.Math.sqrt(dumcoords.dot(dumcoords)); // use theta to store
                                                         //effective length squared of new
                                                         //vector 
    if(theta>1.0) dumcoords.scalarmult(1.0/theta);      // if too big normalize
    dumdiisp.setDiisp(dumcoords.converttoDiisp(perspective));// convert back to Diisp
    globalu=dumdiisp.h;
    globalv=dumdiisp.v;
    
    if(a.ilive){
      a.setDiispV(globalu,globalv);
      c.updatePV(a,b);
      repaint();
    }
    if(b.ilive){
      b.setDiispV(globalu,globalv);
      c.updatePV(a,b);
      repaint();
    }
  }  
  else{// rotate the whole construction
    phi=(double)(u-oldgu)*PI/(double)perspective.r.height/2.0;
    theta=(double)(v-oldgv)*PI/(double)perspective.r.height/2.0;
    a.rotateV(theta,phi);
    b.rotateV(theta,phi);
    c.rotateV(theta,phi);
    grid.rotateP(theta,phi);
    repaint();
  }
  oldgv=v;
  oldgu=u;
  if(angbox!=null) angbox.AngleDisplayUpdate(a,b,c);
  sendData(2,u,v);
  return true;
}
public Color colorfromstring(String s,Color dfault){
                                 // takes a string (assuming triplet of hex pairs
                                 //returns a color
  Integer i;
  try{
    i=Integer.valueOf(s,16);
    return new Color(i.intValue());
  }
  catch(NumberFormatException e){
    return dfault;
  }
}

   public void sendData(int tag, int u, int v) {

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		DataOutputStream D = new DataOutputStream(out);
		try { D.writeInt(tag); }
		 catch(IOException e){}
	
		try { D.writeInt(u); }
		 catch(IOException e){}

		try { D.writeInt(v); }
		 catch(IOException e){}

		byte[] buffer = out.toByteArray();

		AppEventMessage em = new AppEventMessage(Const.DATA,buffer);
		send(em);
  }

public void receive (AppEventMessage em) {
  int tag,u,v;

  try {
     if (em.getType() == Const.CONTROL) {
		CAppAnswerEventMessage msg =  (CAppAnswerEventMessage) em;
                if (msg.getAnswerType() == CAConsts.ISMASTER) {
			Master = msg.isMaster();
			if (Master) System.out.println ("I am master");
			else System.out.println ("I am slave");
		}
		return;
      }

	  ByteArrayInputStream BI = new ByteArrayInputStream(em.getData());
	  DataInputStream in = new DataInputStream(BI);

	 tag = in.readInt();
	 u = in.readInt(); 
	 v = in.readInt(); 
	
	  if (tag == 0) {
  		int dista,distb;
  		dista=(u-a.dend2.h)*(u-a.dend2.h)+(v-a.dend2.v)*(v-a.dend2.v);
  		distb=(u-b.dend2.h)*(u-b.dend2.h)+(v-b.dend2.v)*(v-b.dend2.v);
  		if(dista<=distb&dista<100) a.waxonV(); // if close to either vector turn it on
  		if(distb<dista&distb<100) b.waxonV();
		oldgu=u;
		oldgv=v;
  		repaint();
		return;
	  }

	  if (tag == 1) {
		  a.waxoffV();  // turn off both vectors
  		  b.waxoffV();
  		  repaint();
  		  buttons.update(a,b,c);
  		  return;
	  }

	  if (tag == 2) {
		double theta,phi;

  		if(a.ilive|b.ilive){  // update the active one
   		 	Diisp dumdiisp= new Diisp(u,v);
    			Coords dumcoords = new Coords(dumdiisp.converttoCoords(perspective,a.normal));
    
    			theta=java.lang.Math.sqrt(dumcoords.dot(dumcoords)); // use theta to store
                                                         //effective length squared of new
                                                         //vector 
    			if(theta>1.0) dumcoords.scalarmult(1.0/theta);      // if too big normalize
    			dumdiisp.setDiisp(dumcoords.converttoDiisp(perspective));// convert back to Diisp
    			globalu=dumdiisp.h;
    			globalv=dumdiisp.v;
    
    			if(a.ilive){
      				a.setDiispV(globalu,globalv);
      				c.updatePV(a,b);
      				repaint();
    			}
    			if(b.ilive){
      				b.setDiispV(globalu,globalv);
      				c.updatePV(a,b);
      				repaint();
    			}
  		}  
  		else{// rotate the whole construction
    			phi=(double)(u-oldgu)*PI/(double)perspective.r.height/2.0;
    			theta=(double)(v-oldgv)*PI/(double)perspective.r.height/2.0;
    			a.rotateV(theta,phi);
    			b.rotateV(theta,phi);
    			c.rotateV(theta,phi);
    			grid.rotateP(theta,phi);
    			repaint();
  		}
  		oldgv=v;
  		oldgu=u;
  		if(angbox!=null) angbox.AngleDisplayUpdate(a,b,c);
         }
 }
 catch (Exception e) {e.printStackTrace();}
}

}/* End of class  Cross*/























































































































