package kepler;

import kepler.view.*;
import kepler.object.*;
import kepler.control.*;
import utils.d2.*;
import java.awt.Dimension;
import utils.timers.*;
import java.awt.*;
import utils.GUI.GridBagHelpers;
import utils.GUI.Bar;

import consts.*;
import messages.*;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.*;

/***********************************************************************
 *  a tangoized version of the kepler applet.
 *  relevant areas can be located by searching for TANGO
 *  this class, and EventHandles should be all that are necessary to
 *    understand for the port.
 ***********************************************************************/


public class TangoKepApp extends interfaces.AppletBase 
implements kepler.EventHandles{
  
  KeplerView view; // large black area where things are drawn
   
  ConfigPanel config;    // these three are the control panels
  GeneralPanel general;  // on right of applet
  SweepPanel sweep;
  
  TimeAreaView taview;  // display of time area pairs

  Planet earth;
  Star sun;
  TwoPerspective p;  // used in drawing the planets

  FixedKicker time;           // provides simulation time
  AnimationThread film,ofilm; // force periodic update of view/taview
  boolean running;
  
  SSystemConfig parameters; // hold on to the parameters defining the orbit

  //TANGO protocol  
  boolean master = false; 
  boolean init_completed = false;
  boolean master_init_handled = false;
  boolean incoming_event = false;

  public final byte TP_START_SWEEPING=0;
  public final byte TP_STOP_SWEEPING=1;
  public final byte TP_PAUSE=2;
  public final byte TP_CONTINUE=3;
  public final byte TP_ERASE_AREAS=4;
  public final byte TP_ERASE_LINES=5;
  public final byte TP_ZOOM_IN=6;
  public final byte TP_ZOOM_OUT=7;
  public final byte TP_SHOW_VEL=8;
  public final byte TP_NO_SHOW_VEL=9;
  public final byte TP_SHOW_ACC=10;
  public final byte TP_NO_SHOW_ACC=11;
  public final byte TP_SHOW_ORB=12;
  public final byte TP_NO_SHOW_ORB=13;
  public final byte TP_SWITCH_CONTROLS=14;
  public final byte TP_MOUSE_DRAG=15;
  public final byte TP_MOUSE_DOWN=16; 
  public final byte TP_SET_MASS=17;
  public final byte TP_SET_SM=18;
  public final byte TP_SET_ECC=19;

  public void init(){
    // TANGO -- register with the server first
    register();

    FixedKicker.dt=0.005;

    int viewW=350;
    int viewH=300;
    setBackground(new Color(60,60,125));
    setForeground(Color.white);

    URL image_base=constructImageBase();
    sun=new Star(new TwoVector(),1.0,image_base);
    earth=new Planet(sun,1.0,0.0,image_base);
    
    parameters=new SSystemConfig();
    parameters.stellarmass=1.0;
    parameters.semimajor=1.0;
    parameters.eccentricity=0.0;

    p=new TwoPerspective( new Dimension(viewW,viewH), 4.0);
    view=new KeplerView(sun,earth,p);
    view.resize(viewW,viewH);
    
    config=new ConfigPanel();
    general=new GeneralPanel();
    sweep=new SweepPanel();
    
    taview=new TimeAreaView(earth);
    myLayout();
    

    running=false;

    init_completed = true;
    if (!master_init_handled) 
    {
      if (master) enableAWT(); 
      else disableAWT();
      master_init_handled = true;
    }
  }

  void myLayout(){
    
    setLayout(new GridBagLayout());
    GridBagHelpers.insert(this,view,
			  1,1,
			  1,4,
			  GridBagConstraints.BOTH,
			  GridBagConstraints.NORTHWEST,
			  0,0,
			  1.0,1.0,
			  8,8,0,0			  
			  );
    GridBagHelpers.insert(this,general,
			 2,1,
			  1,1,
			 GridBagConstraints.VERTICAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  0.0,1.0,
			  8,0,5,8
			  );
    GridBagHelpers.insert_fill(this,new Bar(),
			       2,2,
			       1,1,
			       5,0,5,8
			       );
    GridBagHelpers.insert(this,config,
			  2,3,
			  1,1,
			  GridBagConstraints.VERTICAL,
			  GridBagConstraints.CENTER,
			  0,0,
			  0.0,2.0,
			  5,5,5,8
			  );
    GridBagHelpers.insert(this,new Bar(),
			  2,4,	
			  1,1,
			  GridBagConstraints.HORIZONTAL,
			  GridBagConstraints.SOUTH,
			  0,0,
			  0.0,0.0,
			  5,0,5,8
			  );
    
    
    GridBagHelpers.insert(this,taview,
			  1,5,
			  1,1,
			  10,8,8,5,
			  GridBagConstraints.CENTER
			  );
    GridBagHelpers.insert(this,sweep,
			  2,5,
			  1,1,
			  10,5,8,8,
			  GridBagConstraints.CENTER
			  );
  }
  


  void startThreads(){
    if(!running){
      film=new AnimationThread(view,75);
      ofilm=new AnimationThread(taview,75);
      time=new FixedKicker(earth,75);
      time.start();
      film.start();
      ofilm.start();
      running=true;
    }
  }
  void stopThreads(){
    if(running){
      film.pleaseStop();
      ofilm.pleaseStop();
      time.pleaseStop();
      running=false;
    }
  }


  
  public boolean action(Event e,Object arg){
    if( arg instanceof String) 
    {
      if ( master ) tangoizeEvent(e,0,0);
      return stringAction(e,(String)arg);
    }
    else if(arg instanceof SSystemConfig){
      stopThreads();
      handleEvent(new Event(this,Event.ACTION_EVENT,STOP_SWEEPING));
      distributeEvent(new Event(this,Event.ACTION_EVENT,ERASE_AREAS));
      SSystemConfig ss=(SSystemConfig)arg;
      sun.setMass(ss.stellarmass);
      earth.defineOrbit(sun,ss.semimajor,ss.eccentricity);
      if ( master )
      {
	sendNewMass(ss.stellarmass);
	sendNewSemiMajor(ss.semimajor);
	sendNewEcc(ss.eccentricity);
      }
      return distributeEvent(e);
    }
    else return super.action(e,arg);
  }
  
  boolean stringAction(Event e,String sarg){
    // can get signals which affect threads
    if(PAUSE.equals(sarg) ){ stopThreads();  return true; }
    else if(CONTINUE.equals(sarg) ){  startThreads(); return true; }
    // if sarg starts with "Zoom" -- then it affects the 
    // view's perspective so send it there
    else if( sarg.startsWith("Zoom") ||
             sarg.equals(SHOW_VEL)   ||
             sarg.equals(NO_SHOW_VEL)||
	     sarg.equals(SHOW_ACC)   ||
             sarg.equals(SHOW_ORB)   ||
	     sarg.equals(NO_SHOW_ORB)||
             sarg.equals(NO_SHOW_ACC) )   return view.handleEvent(e);
    else if( START_SWEEPING.equals(sarg)){
      earth.startSweeping();
      view.readEvent(e);
      sweep.readEvent(e);
      taview.readEvent(e);
      return true;
    }
    else if( STOP_SWEEPING.equals(sarg)){
      earth.stopSweeping();
      view.readEvent(e);
      sweep.readEvent(e);
      taview.readEvent(e);
      return true;
    }
    else if(ERASE_AREAS.equals(sarg)){
      earth.stopSweeping();
      boolean tmp=distributeEvent(e);
      if(!film.isAlive()) view.draw();
      return tmp;
    }
    else if( ERASE_LINES.equals(sarg)){
      boolean tmp=distributeEvent(e);
      if( !running ) view.draw();
      return tmp;
    }
    else return false;
  }
    

  public boolean mouseDown(Event e,int x,int y){
    if( e.target==view && !running ){
      view.draw();
      if (!(e.arg instanceof SSystemConfig || incoming_event)) tangoizeEvent(e,x,y);
      return true;
    }
    else return super.mouseDown(e,x,y);
  }
  
  public boolean mouseDrag(Event e,int x,int y){
    if( e.target==view && !running){
      view.draw();
      if (!(e.arg instanceof SSystemConfig || incoming_event)) tangoizeEvent(e,x,y);
      return true;
    }
    else return super.mouseDrag(e,x,y);
  }

  boolean distributeEvent(Event e){
    boolean tmp=true;
    tmp=view.readEvent(e)|taview.readEvent(e)|
      general.readEvent(e)|sweep.readEvent(e)|config.readEvent(e);
    return tmp;
  }

  public boolean readEvent(Event e){
    return handleEvent(e)||distributeEvent(e);
  }
  
  public void stop(){ stopThreads(); }


  URL constructImageBase(){
    URL tmp=null;
    String arg=getParameter("image_offset");
    try{
        
      if( arg != null){
	tmp=new URL(getCodeBase(),arg);
      }
      else{
	tmp=new URL(getCodeBase(),"kepler/images/");
      }
    }
    catch(MalformedURLException mue){
      System.out.println("unable to locate images...");
      System.out.println("code_base "+getCodeBase());
      System.out.println("supplied offset "+arg);
      System.out.println("default offset "+"kepler/images/");
      return null;
    }
    return tmp;
  }


 

  /***************************************************
   *  TANGO 
   *  first look at recieving messages
   ***************************************************/


  public void receive(AppEventMessage m){
    if( m.getType() == Const.CONTROL ) receiveControl(m);
    else                               receiveData(m);
  }

  void receiveControl(AppEventMessage m){
    CAppAnswerEventMessage mes =  (CAppAnswerEventMessage) m;

    switch((int) mes.getAnswerType())
      {
        case CAConsts.ISMASTER: master = mes.isMaster();
	                        if (master) 
				{
				  if (init_completed) 
				  {
				    enableAWT();
				    master_init_handled = true;
				  }
				}
				else 
				{
				  if (init_completed) 
				  {
				    disableAWT();
				    master_init_handled = true;
				  }
				}
				break;
      }			
  }

  void receiveData(AppEventMessage m){
    byte[] tab = m.getData();
    ByteArrayInputStream bis = new ByteArrayInputStream(tab);
    switch((int) bis.read())
      {
	case TP_START_SWEEPING: stringAction(new Event( this, Event.ACTION_EVENT, START_SWEEPING), START_SWEEPING);
	  break;

	case TP_STOP_SWEEPING:  stringAction(new Event( this, Event.ACTION_EVENT, STOP_SWEEPING), STOP_SWEEPING); 
	  break;
	case TP_PAUSE:  stringAction(new Event( this, Event.ACTION_EVENT, PAUSE), PAUSE); 
	  break;
	case TP_CONTINUE:  stringAction(new Event( this, Event.ACTION_EVENT, CONTINUE), CONTINUE); 
	  break;
	case TP_ERASE_AREAS:  stringAction(new Event( this, Event.ACTION_EVENT, ERASE_AREAS), ERASE_AREAS);  
	  break;
	case TP_ERASE_LINES:  stringAction(new Event( this, Event.ACTION_EVENT, ERASE_LINES), ERASE_LINES); 
	  break;
	case TP_ZOOM_IN:  stringAction(new Event( this, Event.ACTION_EVENT, ZOOM_IN), ZOOM_IN); 
	  break;
	case TP_ZOOM_OUT:  stringAction(new Event( this, Event.ACTION_EVENT, ZOOM_OUT), ZOOM_OUT); 
	  break;
	case TP_SHOW_VEL:  stringAction(new Event( this, Event.ACTION_EVENT, SHOW_VEL),SHOW_VEL ); 
	  break;
	case TP_NO_SHOW_VEL:  stringAction(new Event( this, Event.ACTION_EVENT, NO_SHOW_VEL), NO_SHOW_VEL); 
	  break;
	case TP_SHOW_ACC:  stringAction(new Event( this, Event.ACTION_EVENT, SHOW_ACC), SHOW_ACC); 
	  break;
	case TP_NO_SHOW_ACC:  stringAction(new Event( this, Event.ACTION_EVENT, NO_SHOW_ACC), NO_SHOW_ACC); 
	  break;
	case TP_SHOW_ORB:  stringAction(new Event( this, Event.ACTION_EVENT, SHOW_ORB), SHOW_ORB); 
	  break;
	case TP_NO_SHOW_ORB:  stringAction(new Event( this, Event.ACTION_EVENT, NO_SHOW_ORB), NO_SHOW_ORB); 
	  break;
	case TP_SWITCH_CONTROLS:  stringAction(new Event( this, Event.ACTION_EVENT, SWITCH_CONTROLS), SWITCH_CONTROLS); 
	  break;
	case TP_MOUSE_DRAG: 
	  DataInputStream dis = new DataInputStream(bis);
	  try
	  {
	    int x = dis.readInt();
	    int y = dis.readInt();
	    incoming_event = true;
	    distributeEvent(new Event( view, 0, Event.MOUSE_DRAG, x, y, 0, 0)); 
	    incoming_event = false;
	  }
	  catch (Exception ex) {}
	  break;
	case TP_MOUSE_DOWN: 
	  dis = new DataInputStream(bis);
	  try
	  {
	    int x = dis.readInt();
	    int y = dis.readInt();
	    incoming_event = true;
	    distributeEvent(new Event( view, 0, Event.MOUSE_DOWN, x, y, 0, 0));
	    incoming_event = false;
	  }
	  catch (Exception ex) {}
	  break; 
        case TP_SET_MASS:
	  dis = new DataInputStream(bis);
	  try
	  {
	    double d = dis.readDouble();
	    receiveNewMass(d); 
	  }
	  catch (Exception ex) {}
	  break;
        case TP_SET_SM:
	  dis = new DataInputStream(bis);
	  try
	  {
	    double d = dis.readDouble();
	    recieveNewSemiMajor(d); 
	  }
	  catch (Exception ex) {} 
	  break;
        case TP_SET_ECC:
	  dis = new DataInputStream(bis);
	  try
	  {
	    double d = dis.readDouble();
	    recieveNewEcc(d); 
	  }
	  catch (Exception ex) {}  
	  break;
      }
  }




  
  /****************************************
   * TANGO -- here do the sending part
   ****************************************/

  void tangoizeEvent(Event e, int x, int y){
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
 
    if (e.id == Event.ACTION_EVENT)
    {
      if ( e.arg.equals(SHOW_ACC)) bos.write(TP_SHOW_ACC);
      else if ( e.arg.equals(START_SWEEPING)) bos.write(TP_START_SWEEPING);
      else if ( e.arg.equals(STOP_SWEEPING)) bos.write(TP_STOP_SWEEPING);
      else if ( e.arg.equals(PAUSE)) bos.write(TP_PAUSE);
      else if ( e.arg.equals(CONTINUE)) bos.write(TP_CONTINUE);
      else if ( e.arg.equals(ERASE_AREAS)) bos.write(TP_ERASE_AREAS);
      else if ( e.arg.equals(ERASE_LINES)) bos.write(TP_ERASE_LINES);
      else if ( e.arg.equals(ZOOM_IN)) bos.write(TP_ZOOM_IN);
      else if ( e.arg.equals(ZOOM_OUT)) bos.write(TP_ZOOM_OUT);
      else if ( e.arg.equals(SHOW_VEL)) bos.write(TP_SHOW_VEL);
      else if ( e.arg.equals(NO_SHOW_VEL)) bos.write(TP_NO_SHOW_VEL);
      else if ( e.arg.equals(SHOW_ORB)) bos.write(TP_SHOW_ORB);
      else if ( e.arg.equals(NO_SHOW_ORB)) bos.write(TP_NO_SHOW_ORB);
      else if ( e.arg.equals(SWITCH_CONTROLS)) bos.write(TP_SWITCH_CONTROLS);
      else if ( e.arg.equals(NO_SHOW_ACC)) bos.write(TP_NO_SHOW_ACC);
    }
    else if (e.id == Event.MOUSE_DOWN)
    {
      bos.write(TP_MOUSE_DOWN);
      DataOutputStream dos = new DataOutputStream(bos);
      try 
	{
	  dos.writeInt(e.x);
	  dos.writeInt(e.y);
	} 
      catch (Exception ex) {}
    }
    else if (e.id == Event.MOUSE_DRAG)  
    {
      bos.write(TP_MOUSE_DRAG);
      DataOutputStream dos = new DataOutputStream(bos);
      try 
	{
	  dos.writeInt(e.x);
	  dos.writeInt(e.y);
	} 
      catch (Exception ex) {}
    }
    
    AppEventMessage am = new AppEventMessage(Const.DATA,bos.toByteArray());
    send(am);
  }


  /*************************************
   * TANGO -- the functions which you have requested

   * 
   *************************************/

  public void sendNewMass(double new_mass){
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    bos.write(TP_SET_MASS);
    DataOutputStream dos = new DataOutputStream(bos);
    try 
      {
	dos.writeDouble(new_mass);
      } 
    catch (Exception ex) {}
    AppEventMessage am = new AppEventMessage(Const.DATA,bos.toByteArray());
    send(am);
  }

  public void sendNewSemiMajor(double semi){
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    bos.write(TP_SET_SM);
    DataOutputStream dos = new DataOutputStream(bos);
    try 
      {
	dos.writeDouble(semi);
      } 
    catch (Exception ex) {}
    AppEventMessage am = new AppEventMessage(Const.DATA,bos.toByteArray());
    send(am); 
  }

  public void sendNewEcc(double ecc){
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    bos.write(TP_SET_ECC);
    DataOutputStream dos = new DataOutputStream(bos);
    try 
      {
	dos.writeDouble(ecc);
      } 
    catch (Exception ex) {}
    AppEventMessage am = new AppEventMessage(Const.DATA,bos.toByteArray());
    send(am);
  }


  public void receiveNewMass(double m){
    parameters.stellarmass=m;
    setNewConfiguration(parameters);
  }
  public void recieveNewSemiMajor(double semi){
    parameters.semimajor=semi;
    setNewConfiguration(parameters);
  }
  public void recieveNewEcc(double ecc){
    parameters.eccentricity=ecc;
    setNewConfiguration(parameters);
  }


  void setNewConfiguration(SSystemConfig sc){
    stopThreads();
    handleEvent(new Event(this,Event.ACTION_EVENT,STOP_SWEEPING));
    distributeEvent(new Event(this,Event.ACTION_EVENT,ERASE_AREAS));
    distributeEvent( new Event( this,Event.ACTION_EVENT,sc) );
    sun.setMass(sc.stellarmass);
    earth.defineOrbit(sun,sc.semimajor,sc.eccentricity);
  }

  void disableAWT(){
    config.disable();
    general.disable();
    sweep.disable();
    //view.disable();
  }
  
  void enableAWT(){
    config.enable();
    general.enable();
    sweep.enable();
    //view.enable();
  }

  
}










