//package dev.metin.OSC36;
//
// Note that the help images are stored along with the class files
// and thus the path from the codebase to the images is hardcoded
// to be the same as the package name in Oscillator.init().
//

import java.applet.Applet;
import java.awt.*;
import java.util.*;
import java.awt.Color;
//import EDU.syr.phy.mra.Style;
//import EDU.syr.phy.mra.HelpWindow;
 
///////// Tango API ////////
import interfaces.AppletBase;
import messages.*;
import consts.Const;
import CAConsts;

public class Oscillator extends AppletBase {
    public RFramedArea framedArea;
    public Controls cont; 
    Image helpImage;
    int md;
// Master-slave status
  boolean            Master = true;

    public void init() {

	//////// Register to Tango (this must be the first statement) ////
  	register();

	String s = getParameter("mode");                          // Get mode paramater
	md = ((s == null) ? 1 : Integer.valueOf(s).intValue());   // and load the     
	System.out.println("Mode " + md);



	if (md==1) { 
	  helpImage=getImage(getCodeBase(),"./help1.gif"); 
	} else if (md==2) {
	  helpImage=getImage(getCodeBase(),"./help2.gif");
	} else {
	  helpImage=(Image)null;
	}
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
 
        setLayout(gridBag);
 
        framedArea = new RFramedArea(this);
        cont = new Controls(this);
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1.0;
        c.gridwidth = GridBagConstraints.REMAINDER; //end row
        gridBag.setConstraints(framedArea, c);
        add(framedArea);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.weighty = 0.0;
        gridBag.setConstraints(cont, c);
        add(cont);
 
        //Add Components to the Applet, using the default FlowLayout. 
        c.fill = GridBagConstraints.NONE;
        c.weightx = 0.0;
        c.weighty = 0.0;

        validate();
    }

public void receive (AppEventMessage em) {
  try {

     if (em.getType() == Const.CONTROL) {
                CAppAnswerEventMessage msg =  (CAppAnswerEventMessage) em;
                if (msg.getAnswerType() == CAConsts.ISMASTER) {
                        Master = msg.isMaster();
                        if (Master) System.out.println ("I am master");
                        else System.out.println ("I am slave");
                }
                return;
      }
       String msg=new String(em.getData(),0,0,em.getLen());
        String tag = msg.substring(0,1);
        msg = msg.substring(1);
        if (tag.equals("0")) { // clear button
		Event e = new Event(cont.b3,0,null);
		cont.tango_action(e,null);	
	}
	else if (tag.equals("1")) { // draw all button
		Event e = new Event(cont.b4,0,null);
		cont.tango_action(e,null);		
	}
	else if (tag.equals("2")) { // help button
		Event e = new Event(cont.b5,0,null);
		cont.tango_action(e,null);	
	}
	else if (tag.equals("3")) { // mouse down
		int i = msg.indexOf(","); 
		int x = Integer.parseInt(msg.substring(0,i));
		int y = Integer.parseInt(msg.substring(i+1));
		framedArea.canv.tango_mouseDown(null,x,y);
	}
	else if (tag.equals("4")) { // mouse up
		framedArea.canv.tango_mouseUp();
	}	
	else if (tag.equals("5")) { // mouse drag
		int i = msg.indexOf(","); 
		int x = Integer.parseInt(msg.substring(0,i));
		int y = Integer.parseInt(msg.substring(i+1));
		framedArea.canv.tango_mouseDrag(null,x,y);
	}
 }
 catch (Exception e) {e.printStackTrace();}

}

}


class RFramedArea extends Panel {
    public OscCanvas canv;
    public RFramedArea(Oscillator controller) {
        super();
 
        //Set layout to one that makes its contents as big as possible.
        setLayout(new GridLayout(1,1));
        canv = new OscCanvas(controller);
        add(canv);
        validate();
        }
 
        public Insets insets() {
        return new Insets(4,4,5,5);
        }
 
    public void paint(Graphics g) {
	Dimension d = size();
        Color bg = getBackground();
        g.setColor(bg);
        g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
        g.draw3DRect(3, 3, d.width - 7, d.height - 7, false);
    }
  }

class Controls extends Panel {
    public Button b3,b4,b5; //Buttons for controls.
    public Oscillator ref;
    HelpWindow help;
    Style S;
    String helpString = "This applet is designed to illustrate how the " +
      "force acting on a spring and the energy stored in the spring " +
      "change with extension.\n\n" +
      "The simplest way to use the applet is to drag the Mass away from " +
      "its equilibrium point. You will see that curves are traced out " +
      "on both graphs above. The left graph shows how the force (F) " +
      "applied to the mass increases as the spring is stretched " +
      "(increasing X). The right graph shows how the energy (E) increases " +
      "as the spring is stretched.\n\n" +
      "Now, if the mass is moved left of the equilibrium the spring is " +
      "compressed (X is negative). We see that the force changes sign " +
      "yet the energy increases again - energy is stored in the squashed " +
      "spring.\n\n" +
      "--\n" +
      "Applet written by Metin Sezgin and modified by Simeon Warner\n" +
      "Last modified February 1997\n";
      
    public Controls(Oscillator controller) {
        super();
        //Define leftControls and rightControls
        Panel leftControls = new Panel();
        leftControls.setLayout(new FlowLayout());
        b3 = new Button();
        b3.setLabel("Clear");
        b4 = new Button();
        b4.setLabel("Draw All");
        leftControls.add(b3);
        leftControls.add(b4);
        
        Panel rightControls = new Panel();
        rightControls.setLayout(new FlowLayout());
        b5 = new Button();
        b5.setLabel("Help");
        rightControls.add(b5); 
        // Add two sections at left and right sides, and blank center.
        setLayout(new BorderLayout());
        add("West",leftControls);
        add("Center",new Panel());
        add("East",rightControls);
        
        ref=controller;
        validate();
	
      }

    public Insets insets() {
        return new Insets(4,4,5,5);
      }
    
    public boolean action(Event e, Object arg) {
	if (!ref.Master && e.target != b5) return true;
	return (tango_action(e,arg));
    }
    public  boolean tango_action(Event e, Object arg) {
        Object target = e.target;
	if (target == b3) {
	    ref.framedArea.canv.m.l_bound = 
	      ( ref.framedArea.canv.m.location.width -
	       ref.framedArea.canv.min+5)/2;
	    ref.framedArea.canv.m.u_bound = 
                        ( ref.framedArea.canv.m.location.width -
			 ref.framedArea.canv.min+5)/2;
	    ref.framedArea.canv.repaint();
	    sendmsg("0");
	    return true;
	  }
	if (target == b4) {
	    ref.framedArea.canv.m.l_bound = 0;
	    ref.framedArea.canv.m.u_bound = 139;
	    ref.framedArea.canv.repaint();
  	    sendmsg("1");
	    return true;
	  }
	if (target == b5) {
	    Dimension dim = new Dimension(50,10);
	    help = new HelpWindow(ref.helpImage,
	                          "Help for Oscillator Applet",
	                          helpString,dim);
	    help.repaint();
	    help.show();
	    sendmsg("2");
	    return true;
	  }
	return false;
      }

    public void sendmsg(String msg) {
	if (!ref.Master) return;
        try {
          byte[] t=new byte[msg.length()];
          msg.getBytes(0,msg.length(),t,0);
          AppEventMessage em=new AppEventMessage(Const.DATA,t);
          ref.send(em);
        }
        catch (Exception e) {e.printStackTrace();}
}


    public void paint(Graphics g) {
	Dimension d = size();
        Color bg = getBackground();
        g.setColor(bg);
      }
  }

// This class represents a physical block with a  location, 
// old_location dimensions, upper and lower bounds.

class Block{
Dimension old_location = new Dimension(100,100);
Dimension location = new Dimension(100,100);
Dimension dim = new Dimension(40,40);

int l_bound=0;
int u_bound=0;

Block(){
location.width=100;
location.height=360;
dim.width=40;
dim.height=40;
}

Block(int x,int y,int w,int h){
old_location.width=x;
old_location.height=y;
location.width=x;
location.height=y;
dim.width=w;
dim.height=h;
}

//An alternative constructor.

Block(int x,int y,int w,int h,int lb, int ub){
old_location.width=x;
old_location.height=y;
location.width=x;
location.height=y;
l_bound=lb;
u_bound=ub;
dim.width=w;
dim.height=h;
}

// This grab function returns true if the point representet by the 
// x and y parameters are inside it. Else it returns false.
boolean grab(int x, int y){
  if ((x<(location.width)) && (x>(location.width-dim.width)) &&
     (y<(location.height)) && (y>(location.height-dim.height)))
  return true;
  else return false;
  }

}

// This class represents tracers. We use them in order to show where
// on a curve we are. 
class Tracer{
Dimension location = new Dimension(100,100);
int radius;
boolean draggable=false;
int x_min, x_max;

Tracer(){
location.width=0;
location.height=0;
radius=0;
draggable=false;
x_min=0;
x_max=0;
}

Tracer(int x,int y){
location.width=x;
location.height=y;
radius=5;
draggable=false;
x_min=0;
x_max=2*x;
}

 Tracer(int x,int y,int xmin,int xmax){
 location.width=x;
 location.height=y;
 radius=5;
 draggable=false;
 x_min=xmin;
 x_max=xmax;
 }
// This grab function is works similar to that of a Block object.
boolean grab(int x, int y){
  if ((x<(location.width+radius)) && (x>(location.width-radius)) &&
     (y<(location.height+radius)) && (y>(location.height-radius)))
  return true;
  else return false;
  }

}

// A new class. This represents a wall kind of thing where spring is
// connected. It has a location and a dimension. It also has its own
// drawing function. This function needs to be passes a Graphics 
// object to draw on.

class Base {
 Dimension loc = new Dimension(0,0);
 Dimension dim = new Dimension(10,10);

 Base(int x, int y) {
 this.dim = new Dimension(x,y);
}

Base(int x1, int y1, int x2, int y2) {
 this.dim = new Dimension(x1,y1);
 this.loc = new Dimension(x2,y2);
}

public void draw(Graphics g, double slope) {
 int i=0, x1=0, x2=0, y1=0, y2=0, n=0;
 double dx, dy;

 n=dim.width/3;
 dx=((double)dim.width)/((double)n);
 dy=((double)dim.height)/((double)n);

 for (i=0; i<=n; ++i)
 {
   x1= loc.width+(int)(dx*i);
   x2= loc.width;
   y1= loc.height;
   y2= loc.height+(int)(dy*i);
   g.drawLine(x1,y1,x2,y2);
   g.drawLine(loc.width+dim.width-(int)(dx*i) , loc.height+dim.height,
              loc.width+dim.width , loc.height+dim.height-(int)(dy*i));
 }

 g.drawLine(loc.width+dim.width+1,loc.height,loc.width+dim.width+1,loc.height+dim.height);	
}

}

