// Style.java
// Simeon Warner, Metin Sezgin
// Last updated: see @version

//package EDU.syr.phy.mra;

import java.util.Observable;
import java.awt.Font;
import java.awt.Color;
import java.awt.Dimension;

/** This class will be used for all MRA work in the Syracuse Physics
*   deparment to provide a uniform look to the applets generated. 
* 
*@author Simeon Warner
*@author Metin Sezgin
*@version 14Feb97
*/
public class Style extends Observable {

// Colors - for diagrams and graphs
  public static Color bgDiagramColor = Color.white;
  public static Color fgDiagramColor = Color.blue;
  public static Color fg2DiagramColor = Color.blue.brighter();
  public static Color axesColor = Color.black;
  public static Color labelsColor = Color.black;
  public static Color errorColor = Color.red;
// Colors - for control and data entry panels
  public static Color bgControlColor = Color.lightGray;
  public static Color fgControlColor = Color.black;
  public static Color highlightedColor = Color.red;
  public static Color selectedColor = Color.blue;
// Shading parameters
  public static Color darkestShade = new Color(100,100,100);
  public static Color lightestShade = new Color(240,240,240);
  public static int numShades = 32;
  private static Color[] shadeColor;
  private static int nShades;

// Fonts for labels 
  public static int fSize = 12;
  public static Font labelsFont = new Font("Courier",0,fSize);
  public static Font labels2Font = new Font("Courier",0,11);

// Fonts for controls
  public static Font controlFont = new Font("Courier",0,fSize);

// Fonts for helpWindow
  public static Font helpFont = new Font("NewYork",0,fSize+2);


// Window style parameters
  public static int frameWidth = 4;
  public static int outsideBorderWidth = 5;

// HelpWindow parameters
  public static Dimension helpWindowDim = new Dimension(320,220);

  
  /** This routine must be called before the getShade() function is
      called. It initialises the numShades elements of the array.
  */
  public static void initShade() {
    int j, rd,gd,bd, rl,gl,bl;
    nShades=numShades;
    rd=darkestShade.getRed();
    gd=darkestShade.getGreen();
    bd=darkestShade.getBlue();
    rl=lightestShade.getRed()-rd;
    gl=lightestShade.getBlue()-gd;
    bl=lightestShade.getGreen()-bd;
    shadeColor = new Color[nShades];
    for (j=0; j<nShades; j++) {
      shadeColor[j] = new Color(rd+(int)((rl*j)/(float)nShades),
                                gd+(int)((gl*j)/(float)nShades),
                                bd+(int)((bl*j)/(float)nShades));
    }
  }

  
  public static Color getShade(float aShade) {
  /** This routine returns a Color from the array initialised by initShade()
      based on the input value, aShade. aShade <= 0.0 will return the color
      given by lightestShade, aShade >= 1.0 will return the color given by
      darkestShade, and value in between will  returnone of the numShades 
      colors obtained by linear interpolation.
   */
    int iShade;
    iShade = (int)(aShade*(float)nShades);
    return(shadeColor[((iShade<0) ? 0 : ((iShade>=nShades) ? (nShades-1) : iShade))]);
  }

  
} /***** end of class Style *****/
